# | Copyright 2016 Karlsruhe Institute of Technology
# |
# | Licensed under the Apache License, Version 2.0 (the "License");
# | you may not use this file except in compliance with the License.
# | You may obtain a copy of the License at
# |
# |     http://www.apache.org/licenses/LICENSE-2.0
# |
# | Unless required by applicable law or agreed to in writing, software
# | distributed under the License is distributed on an "AS IS" BASIS,
# | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# | See the License for the specific language governing permissions and
# | limitations under the License.

import os
from grid_control.backends.backend_tools import BackendExecutor
from hpfwk import AbstractError
from python_compat import tarfile

class RetrieveJobs(BackendExecutor):
	def execute(self, wmsIDs): # yields list of (wmsID, local_output_dir)
		raise AbstractError


class RetrieveJobsEmulateWildcard(RetrieveJobs):
	def __init__(self, config, executor):
		RetrieveJobs.__init__(self, config)
		(self._executor, self._wildcard_file) = (executor, 'GC_WC.tar.gz')

	def setup(self, log):
		RetrieveJobs.setup(self, log)
		self._executor.setup(log)

	def execute(self, wmsIDs): # yields list of (wmsID, local_output_dir)
		for (wmsID, local_output_dir) in self._executor.execute(self, wmsIDs):
			if local_output_dir and os.path.exists(local_output_dir):
				fn_wildcard_tar = os.path.join(local_output_dir, self._wildcard_file)
				if os.path.exists(fn_wildcard_tar):
					try:
						tarfile.TarFile.open(fn_wildcard_tar, 'r:gz').extractall(local_output_dir)
					except Exception:
						self._log.error('Unable to unpack output files contained in %s', fn_wildcard_tar)
						continue
					try:
						os.unlink(fn_wildcard_tar)
					except Exception:
						self._log.error('Unable to remove wildcard emulation file %s', fn_wildcard_tar)
			yield (wmsID, local_output_dir)


class RetrieveAndPurgeJobs(RetrieveJobs):
	pass
